use strict;
use XML::Writer;
use IO;

# Open the file provided in the command line argument.
my $input = new IO::File(shift);

# Open the output file.
my $output = new IO::File(">ch5_book_inventory_report.xml");

# Instantiate an XML::Writer object.
my $writer = XML::Writer->new(OUTPUT => $output, DATA_MODE => 1, DATA_INDENT => 2);

# Start generating the XML document.
$writer->xmlDecl("UTF-8");
$writer->doctype("book_inventory", "", "ch5_book_inventory.dtd");
$writer->comment("**** Book Inventory Database ****");
$writer->startTag("book_inventory");

# Create a list of names that match the input CSV column names.
my @column_names = qw(Title Author ISBN Publisher Price Quantity);

# Loop the input file.
while (<$input>) {
  chomp; # chomp the new line char from each line

  # Remove leading and trailing whitespace, split on a ','.
  my @data = map {s/^\s+//; s/\s+$//; $_;} split /,/;
  $writer->startTag("book", "Title" => $data[0]);

  # Loop through the data contained in this row.
  foreach my $index (1..$#column_names) {
    $writer->startTag($column_names[$index]);
    $writer->characters($data[$index]);
    $writer->endTag($column_names[$index]);
  }
  $writer->endTag("book");
}

# Don't forget the end tag.
$writer->endTag("book_inventory");

# Destroy the XML::Writer object and close the output file.
$writer->end();
$output->close();
